import asyncio
import json
from typing import override

from easyrip import log
from watchdog.events import FileSystemEvent, FileSystemEventHandler
from watchdog.observers import Observer

from .font import font_data
from .web.msg import process_msg, push_msg


class File_watch(FileSystemEventHandler):
    @override
    def on_any_event(self, event: FileSystemEvent) -> None:
        log.info(
            f'File watch: {event.event_type}: "{event.src_path}"{f' -> "{event.dest_path}"' if event.dest_path else ""}',
            is_format=False,
        )
        font_data.refresh_font_data_dict()
        asyncio.run(push_msg(asyncio.run(process_msg(json.dumps({"get_fonts": ""})))))


async def start_file_watch(*paths: str):
    observer = Observer()
    for path in paths:
        observer.schedule(File_watch(), path, recursive=True)
    observer.start()
    return observer



from easyrip import log
from easyrip.ripper.sub_and_font.font import Font, load_fonts


class font_data:
    font_data_dict: dict[str, list[Font]] = {}
    """实时变化的监听目录及其结果"""

    @classmethod
    def refresh_font_data_dict(cls):
        cls.font_data_dict = {d: load_fonts(d) for d in cls.font_data_dict}

    @classmethod
    def add_font_data_dir(cls, *dirs: str):
        for d in dirs:
            cls.font_data_dict[d] = load_fonts(d)

    @classmethod
    def pop_font_data_dir(cls, *dirs: str):
        for d in dirs:
            try:
                cls.font_data_dict.pop(d)
            except KeyError as e:
                log.error(
                    "{} faild: {}",
                    cls.pop_font_data_dir.__name__,
                    e,
                )

    @classmethod
    def get_font_info(
        cls,
        pathname: str,
    ):  # TODO
        fonts = load_fonts(pathname)



import asyncio
import os

from easyrip import log

from .file_watch import start_file_watch
from .font import font_data
from .web import server


async def main():
    WIN_FONT_PATHS: tuple[str, ...] = (
        os.path.join(os.environ["SYSTEMROOT"], "Fonts"),
        os.path.join(os.environ["LOCALAPPDATA"], "Microsoft/Windows/Fonts"),
    )

    log.write_level = log.LogLevel.none
    log.init()

    font_data.add_font_data_dir(*WIN_FONT_PATHS)

    await asyncio.gather(server.run(), start_file_watch(*WIN_FONT_PATHS))


def run():
    asyncio.run(main())


if __name__ == "__main__":
    run()
